/*******************************************************************************
*    Robot "Giskard" 1.0.                                                      *
*    Copyright (C) 2009-averyfarwaydate Luca Bertoncello                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    Mechanism by Mario Zimmermann   mario@tesla-crew.de                       *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#include <wx/wx.h>

#ifndef GISKARDCONNECTOR_H_
#define GISKARDCONNECTOR_H_

#define SOCKET          int
#define closesocket     close
#define INVALID_SOCKET  (SOCKET)(~0)

/**
 * Klasse für die Verbindung mit dem Roboter über TCP/IP
 */
class GiskardConnector
{
protected:
  char login[255], passwd[255], host[255], error[1024], cameraTmpFile[255], flatTmpFile[255];
  bool connected, tryed;
  SOCKET sock;

  /**
   * Öffnet ein Socket zum Roboter
   *
   * @return bool                     true wenn erfolgreich verbunden, false ansonsten (der Fehler wird angezeigt)
   */
  bool connectSocket(void);

public:
  GiskardConnector();
  virtual
  ~GiskardConnector();

  /**
   * Gibt den Host zurück, an dem das Programm verbunden ist
   *
   * @return char*                    der Host
   */
  char *getHost(void);

  /**
   * Gibt den Pfad zur Datei zurück, wo das aktuelle Bild aus der
   * Kamera gespeichert ist
   *
   * @return wxString                 der Pfad zur Datei
   */
  wxString getImageFromCamera(void);

  /**
   * Gibt den Pfad zur Datei zurück, wo das Bild mit dem Grundriss
   * der Wohnung gespeichert ist
   *
   * @return wxString                 der Pfad zur Datei
   */
  wxString getFlatImage(void);

  /**
   * Gibt die Position des Roboters innerhalb der Wohnung zurück
   *
   * @return wxPoint                  die Koordinaten
   */
  wxPoint getRobotPosition(void);

  /**
   * Verbindet mit dem Roboter
   *
   * @param char* login               der Nutzername
   * @param char* passwd              das Passwort
   * @param char* host                die Adresse des Roboters
   * @return bool                     true wenn erfolgreich verbunden, false ansonsten (der Fehler wird angezeigt)
   */
  bool connectToRobot(char *login, char *passwd, char *host);

  /**
   * Gibt die Loginfehlermeldung zurück
   *
   * @return wxString                 die Fehlermeldung
   */
  wxString getLoginError(void);

  /**
   * Gibt zurück, ob einen Verbindungsversuch stattgefunden hat
   *
   * @return bool                     true wenn hat versucht
   */
  bool hasTryed(void);

  /**
   * Setzt zurück die Verbindungsversuche
   */
  void discardTry(void);

  /**
   * Gibt zurück, ob das Programm mit dem Roboter verbunden ist
   *
   * @return bool                     true wenn verbunden
   */
  bool isConnected(void);

  /**
   * Trennt die Verbindung mit dem Roboter.
   * Schickt ein "LOGOUT" zum Programm und löscht die Dateien mit den Daten
   * der Kamera und des Grundrisses
   *
   * @return bool                     true wenn erfolgreich
   */
  bool disconnectFromRobot(void);

  /**
   * Gibt zurück den Wert der Batterie des Mikrocontrollers
   *
   * @return wxString                 der Wert der Batterie als Prozent
   */
  wxString getUCBattery(void);

  /**
   * Gibt zurück den Wert der Batterie des Laptops
   *
   * @return wxString                 der Wert der Batterie als Prozent
   */
  wxString getBattery(void);

  /**
   * Gibt zurück den Wert des Kompasses
   *
   * @return wxString                 der Wert des Kompasses
   */
  wxString getCompass(void);

  /**
   * Gibt zurück die Entfernung zum nächsten Objekt vor dem Roboter
   * oder "Freier Weg" wenn der Weg frei ist
   *
   * @return wxString                 die Entfernung
   */
  wxString getObjects(void);

  /**
   * Gibt zurück ob der FrontLED eingeschaltet ist
   *
   * @return bool                     true wenn eingeschaltet, false wenn ausgeschaltet
   */
  bool getFrontLED(void);

  /**
   * Schaltet das FrontLED ein oder aus
   *
   * @param bool value                true wenn das LED einzuschalten ist, false wenn auszuschalten
   * @return bool                     true wenn erfolgreich ein-, bzw. ausgeschaltet
   */
  bool setFrontLED(bool value);

  /**
   * Gibt die Richtung der Kamera (Servo) zurück
   *
   * @return int                      die Richtung der Kamera, zwischen 5 und 250
   */
  int getCameraDirection(void);

  /**
   * Bewegt die Kamera (Servo)
   *
   * @param int value                 die Richtung (zwischen 5 und 250)
   * @return bool                     true wenn bewegt
   */
  bool setCameraDirection(int value);
};

#endif /* GISKARDCONNECTOR_H_ */
